@@ -38,7 +38,6 @@ const { getDefaultHighWaterMark } = require('internal/streams/state');
 const assert = require('internal/assert');
 const EE = require('events');
 const Stream = require('stream');
-const internalUtil = require('internal/util');
 const { kOutHeaders, utcDate, kNeedDrain } = require('internal/http');
 const { Buffer } = require('buffer');
 const {
@@ -71,6 +70,7 @@ const {
   hideStackFrames,
 } = require('internal/errors');
 const { validateString } = require('internal/validators');
+const { assignFunctionName } = require('internal/util');
 const { isUint8Array } = require('internal/util/types');
 
 let debug = require('internal/util/debuglog').debuglog('http', (fn) => {
@@ -211,27 +211,6 @@ ObjectDefineProperty(OutgoingMessage.prototype, 'writableCorked', {
   },
 });
 
-ObjectDefineProperty(OutgoingMessage.prototype, '_headers', {
-  __proto__: null,
-  get: internalUtil.deprecate(function() {
-    return this.getHeaders();
-  }, 'OutgoingMessage.prototype._headers is deprecated', 'DEP0066'),
-  set: internalUtil.deprecate(function(val) {
-    if (val == null) {
-      this[kOutHeaders] = null;
-    } else if (typeof val === 'object') {
-      const headers = this[kOutHeaders] = { __proto__: null };
-      const keys = ObjectKeys(val);
-      // Retain for(;;) loop for performance reasons
-      // Refs: https://github.com/nodejs/node/pull/30958
-      for (let i = 0; i < keys.length; ++i) {
-        const name = keys[i];
-        headers[name.toLowerCase()] = [name, val[name]];
-      }
-    }
-  }, 'OutgoingMessage.prototype._headers is deprecated', 'DEP0066'),
-});
-
 ObjectDefineProperty(OutgoingMessage.prototype, 'connection', {
   __proto__: null,
   get: function() {
@@ -256,42 +235,6 @@ ObjectDefineProperty(OutgoingMessage.prototype, 'socket', {
   },
 });
 
-ObjectDefineProperty(OutgoingMessage.prototype, '_headerNames', {
-  __proto__: null,
-  get: internalUtil.deprecate(function() {
-    const headers = this[kOutHeaders];
-    if (headers !== null) {
-      const out = { __proto__: null };
-      const keys = ObjectKeys(headers);
-      // Retain for(;;) loop for performance reasons
-      // Refs: https://github.com/nodejs/node/pull/30958
-      for (let i = 0; i < keys.length; ++i) {
-        const key = keys[i];
-        const val = headers[key][0];
-        out[key] = val;
-      }
-      return out;
-    }
-    return null;
-  }, 'OutgoingMessage.prototype._headerNames is deprecated', 'DEP0066'),
-  set: internalUtil.deprecate(function(val) {
-    if (typeof val === 'object' && val !== null) {
-      const headers = this[kOutHeaders];
-      if (!headers)
-        return;
-      const keys = ObjectKeys(val);
-      // Retain for(;;) loop for performance reasons
-      // Refs: https://github.com/nodejs/node/pull/30958
-      for (let i = 0; i < keys.length; ++i) {
-        const header = headers[keys[i]];
-        if (header)
-          header[0] = val[keys[i]];
-      }
-    }
-  }, 'OutgoingMessage.prototype._headerNames is deprecated', 'DEP0066'),
-});
-
-
 OutgoingMessage.prototype._renderHeaders = function _renderHeaders() {
   if (this._header) {
     throw new ERR_HTTP_HEADERS_SENT('render');
@@ -312,14 +255,14 @@ OutgoingMessage.prototype._renderHeaders = function _renderHeaders() {
   return headers;
 };
 
-OutgoingMessage.prototype.cork = function() {
+OutgoingMessage.prototype.cork = function cork() {
   this[kCorked]++;
   if (this[kSocket]) {
     this[kSocket].cork();
   }
 };
 
-OutgoingMessage.prototype.uncork = function() {
+OutgoingMessage.prototype.uncork = function uncork() {
   this[kCorked]--;
   if (this[kSocket]) {
     this[kSocket].uncork();
@@ -650,7 +593,7 @@ function matchHeader(self, state, field, value) {
       break;
     case 'content-length':
       state.contLen = true;
-      self._contentLength = value;
+      self._contentLength = +value;
       self._removedContLen = false;
       break;
     case 'date':
@@ -664,13 +607,13 @@ function matchHeader(self, state, field, value) {
   }
 }
 
-const validateHeaderName = hideStackFrames((name, label) => {
+const validateHeaderName = assignFunctionName('validateHeaderName', hideStackFrames((name, label) => {
   if (typeof name !== 'string' || !name || !checkIsHttpToken(name)) {
     throw new ERR_INVALID_HTTP_TOKEN.HideStackFramesError(label || 'Header name', name);
   }
-});
+}));
 
-const validateHeaderValue = hideStackFrames((name, value) => {
+const validateHeaderValue = assignFunctionName('validateHeaderValue', hideStackFrames((name, value) => {
   if (value === undefined) {
     throw new ERR_HTTP_INVALID_HEADER_VALUE.HideStackFramesError(value, name);
   }
@@ -678,7 +621,7 @@ const validateHeaderValue = hideStackFrames((name, value) => {
     debug('Header "%s" contains invalid characters', name);
     throw new ERR_INVALID_CHAR.HideStackFramesError('header content', name);
   }
-});
+}));
 
 function parseUniqueHeadersOption(headers) {
   if (!ArrayIsArray(headers)) {
@@ -730,20 +673,22 @@ OutgoingMessage.prototype.setHeaders = function setHeaders(headers) {
   // We also cannot safely split by comma.
   // To avoid setHeader overwriting the previous value we push
   // set-cookie values in array and set them all at once.
-  const cookies = [];
+  let cookies = null;
 
   for (const { 0: key, 1: value } of headers) {
     if (key === 'set-cookie') {
       if (ArrayIsArray(value)) {
+        cookies ??= [];
         cookies.push(...value);
       } else {
+        cookies ??= [];
         cookies.push(value);
       }
       continue;
     }
     this.setHeader(key, value);
   }
-  if (cookies.length) {
+  if (cookies != null) {
     this.setHeader('set-cookie', cookies);
   }
 
@@ -1218,6 +1163,8 @@ OutgoingMessage.prototype._flushOutput = function _flushOutput(socket) {
   // Refs: https://github.com/nodejs/node/pull/30958
   for (let i = 0; i < outputLength; i++) {
     const { data, encoding, callback } = outputData[i];
+    // Avoid any potential ref to Buffer in new generation from old generation
+    outputData[i].data = null;
     ret = socket.write(data, encoding, callback);
   }
   socket.uncork();
