@@ -43,7 +43,7 @@ const nodeTiming = require('internal/perf/nodetiming');
 const timerify = require('internal/perf/timerify');
 const { customInspectSymbol: kInspect, kEnumerableProperty, kEmptyObject } = require('internal/util');
 const { inspect } = require('util');
-const { validateInternalField } = require('internal/validators');
+const { validateThisInternalField } = require('internal/validators');
 const { convertToInt } = require('internal/webidl');
 
 const kPerformanceBrand = Symbol('performance');
@@ -68,7 +68,7 @@ class Performance extends EventTarget {
   }
 
   clearMarks(name = undefined) {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     if (name !== undefined) {
       name = `${name}`;
     }
@@ -77,7 +77,7 @@ class Performance extends EventTarget {
   }
 
   clearMeasures(name = undefined) {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     if (name !== undefined) {
       name = `${name}`;
     }
@@ -85,7 +85,7 @@ class Performance extends EventTarget {
   }
 
   clearResourceTimings(name = undefined) {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     if (name !== undefined) {
       name = `${name}`;
     }
@@ -93,21 +93,24 @@ class Performance extends EventTarget {
   }
 
   getEntries() {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     return filterBufferMapByNameAndType();
   }
 
-  getEntriesByName(name) {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+  getEntriesByName(name, type = undefined) {
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     if (arguments.length === 0) {
       throw new ERR_MISSING_ARGS('name');
     }
     name = `${name}`;
-    return filterBufferMapByNameAndType(name, undefined);
+    if (type !== undefined) {
+      type = `${type}`;
+    }
+    return filterBufferMapByNameAndType(name, type);
   }
 
   getEntriesByType(type) {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     if (arguments.length === 0) {
       throw new ERR_MISSING_ARGS('type');
     }
@@ -116,7 +119,7 @@ class Performance extends EventTarget {
   }
 
   mark(name, options = kEmptyObject) {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     if (arguments.length === 0) {
       throw new ERR_MISSING_ARGS('name');
     }
@@ -124,7 +127,7 @@ class Performance extends EventTarget {
   }
 
   measure(name, startOrMeasureOptions = kEmptyObject, endMark = undefined) {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     if (arguments.length === 0) {
       throw new ERR_MISSING_ARGS('name');
     }
@@ -132,12 +135,12 @@ class Performance extends EventTarget {
   }
 
   now() {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     return now();
   }
 
   setResourceTimingBufferSize(maxSize) {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     if (arguments.length === 0) {
       throw new ERR_MISSING_ARGS('maxSize');
     }
@@ -147,12 +150,12 @@ class Performance extends EventTarget {
   }
 
   get timeOrigin() {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     return getTimeOriginTimestamp();
   }
 
   toJSON() {
-    validateInternalField(this, kPerformanceBrand, 'Performance');
+    validateThisInternalField(this, kPerformanceBrand, 'Performance');
     return {
       nodeTiming: this.nodeTiming,
       timeOrigin: this.timeOrigin,
